# -*- coding: utf-8 -*-
"""
    bromelia.lib.ietf_rfc6733.avps
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module contains the Diameter protocol AVP library that are used 
    to create Diameter base messages.

    :copyright: (c) 2020-present Henrique Marques Ribeiro.
    :license: MIT, see LICENSE for more details.
"""

from ...avps.ietf.rfc6733 import *