# -*- coding: utf-8 -*-
"""
    bromelia.etsi_3gpp_rx
    ~~~~~~~~~~~~~~~~~~~~~

    A Python package with APIs for handling 3GPP Rx Application Id in the
    bromelia Python Library context.

    :copyright: (c) 2020-present Henrique Marques Ribeiro.
    :license: MIT, see LICENSE for more details.
"""