# -*- coding: utf-8 -*-
"""
    bromelia.constants.ietf
    ~~~~~~~~~~~~~~~~~~~~~~~

    This module contains constants defined in IETF specs.
    
    :copyright: (c) 2020-present Henrique Marques Ribeiro.
    :license: MIT, see LICENSE for more details.
"""
