# -*- coding: utf-8 -*-
"""
    bromelia
    ~~~~~~~~

    A Python microframework for building Diameter protocol applications.

    :copyright: (c) 2020-present Henrique Marques Ribeiro.
    :license: MIT, see LICENSE for more details.
"""

from .__version__ import __title__
from .__version__ import __description__
from .__version__ import __url__
from .__version__ import __version__
from .__version__ import __author__
from .__version__ import __author_email__
from .__version__ import __license__
from .__version__ import __copyright__

from .base import *
from .bromelia import Bromelia
from .setup import Diameter