# -*- coding: utf-8 -*-
"""
    bromelia.constants.etsi_3gpp.ts_129_329
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module contains constants defined in ETSI TS 129 329.
    
    :copyright: (c) 2020-present Henrique Marques Ribeiro.
    :license: MIT, see LICENSE for more details.
"""

from ..._internal_utils import convert_to_4_bytes


#: Diameter AVPs
MSISDN_AVP_CODE = convert_to_4_bytes(701)
