# -*- coding: utf-8 -*-
"""
    bromelia.avps.ietf
    ~~~~~~~~~~~~~~~~~~

    This module contains Diameter AVP classes defined in ETSI 3GPP specs.
    
    :copyright: (c) 2020-present Henrique Marques Ribeiro.
    :license: MIT, see LICENSE for more details.
"""
